from fanumbers import consts


def to_fa_number(num, type='persian'):
    """
    :param num: int
    :param type: str, 'persian', 'arabic' or 'all'
    :return: a str if type is not 'all', otherwise a list of all possibilities
    """
    if type not in ["persian", "arabic", "all"]:
        raise ValueError("type should be 'persian', 'arabic' or 'all'")
    nums = str(num)
    res = ''
    if type == "all":
        res = ['']
    for x in nums:
        d = int(x)
        if type == "arabic":
            res += consts.ENGLISH_TO_ARABIC_DIGITS[d]
        elif type == "persian":
            res += consts.ENGLISH_TO_PERSIAN_DIGITS[d]
        else:
            new_res = []
            for i in range(len(res)):
                org = res[i]
                res[i] = org + consts.ENGLISH_TO_ARABIC_DIGITS[d]
                new_res.append(org + consts.ENGLISH_TO_PERSIAN_DIGITS[d])
            res += new_res
    return res


def from_fa_number(num):
    """
    :param num: string, containing arabic and persian digits
    :return: an int
    """
    res = 0
    for x in num:
        d = int(consts.ARABIC_TO_ENGLISH_DIGITS.get(x, consts.PERSIAN_TO_ENGLISH_DIGITS.get(x, None)))
        res = 10 * res + d
    return res



from fanumbers import consts, utils, numbers


def _number_to_words(num, all_possibilities=False):
    if num < 10:
        return [consts.DIGITS_1[num]]
    if num < 100:
        x, y = int(str(num)[0]), int(str(num)[-1])
        if num < 20:
            return [consts.DIGITS_10[y]]
        if y == 0:
            return [consts.DIGITS_10_2[x]]
        return ["{} و {}".format(consts.DIGITS_10_2[x], _number_to_words(y, all_possibilities)[0])]
    if num < 1000:
        x = int(str(num)[1:])
        f = int(str(num)[0])
        res = []
        if x == 0:
            res2 = ['']
        else:
            res2 = [" و " + item for item in _number_to_words(x, all_possibilities)]
        for item in res2:
            if all_possibilities:
                for w in consts.DIGITS_100[f]:
                    res.append(w + item)
            else:
                res.append(consts.DIGITS_100[f][0] + item)
        return res

    if num < int(1e6):
        nums = str(num)
        nums = '0' * (6 - len(nums)) + nums
        x = int(nums[:3])
        y = int(nums[3:])
        res = []
        if y == 0:
            res2 = ['']
        else:
            res2 = [" و " + item for item in _number_to_words(y, all_possibilities)]

        pre = []
        if x == 1:
            pre.append('هزار')
            if all_possibilities:
                pre.append('یک هزار')
        else:
            pre = [item + " هزار" for item in _number_to_words(x, all_possibilities)]
        for f in pre:
            for item in res2:
                res.append(f + item)
        return res
    nums = str(num)
    le = (len(nums) - 1) // 3
    if le > 4:
        raise ValueError("num should be less than 1e15")
    words = ['', '', 'میلیون', 'میلیارد', 'تریلیون']
    lr = le * 3
    ll = len(nums) - lr
    x = int(str(nums)[:ll])
    y = int(str(nums)[ll:])
    pre = [item + " " + words[le] for item in _number_to_words(x, all_possibilities)]
    res = []
    if y == 0:
        res2 = ['']
    else:
        res2 = [" و " + item for item in _number_to_words(y, all_possibilities)]
    for f in pre:
        for item in res2:
            res.append(f + item)
    return res


def number_to_words(num, ordinal=False, all_possibilities=False):
    """
    :param num: an int, or a str containing arabic and persian digits, only works for < 1e15
    :param ordinal: a boolean, False for cardinal and True for ordinal
    :param all_possibilities: it can be True to return all possibilities
    :return: a word, or a list of words in case all_possibilities = True
    """
    if not isinstance(num, int):
        num = numbers.from_fa_number(num)
    if ordinal:
        if num == 0:
            return utils.to_result('صفرم', all_possibilities)
        if num == 1:
            if all_possibilities:
                return ['یکم', 'اول']
            else:
                return 'یکم'
    res = _number_to_words(num, all_possibilities)
    if ordinal:
        ans = []
        for item in res:
            ans.append(item + "م")
        if all_possibilities:
            return ans
        else:
            return ans[0]
    if all_possibilities:
        return res
    return res[0]




ARABIC_DIGITS_LIST = ['٠', '١', '٢', '٣', '٤', '٥', '٦', '٧', '٨', '٩']
PERSIAN_DIGITS_LIST = ['۰', '۱', '۲', '۳', '۴', '۵', '۶', '۷', '۸', '۹']


def list_to_dict(ls, rev=False):
    res = {}
    for i, val in enumerate(ls):
        if rev:
            res[val] = i
        else:
            res[i] = val
    return res


ENGLISH_TO_ARABIC_DIGITS = list_to_dict(ARABIC_DIGITS_LIST)
ARABIC_TO_ENGLISH_DIGITS = list_to_dict(ARABIC_DIGITS_LIST, True)
ENGLISH_TO_PERSIAN_DIGITS = list_to_dict(PERSIAN_DIGITS_LIST)
PERSIAN_TO_ENGLISH_DIGITS = list_to_dict(PERSIAN_DIGITS_LIST, True)

DIGITS_1 = ['صفر', 'یک', 'دو', 'سه', 'چهار', 'پنج', 'شش', 'هفت', 'هشت', 'نه']
DIGITS_10 = ['ده', 'یازده', 'دوازده', 'سیزده', 'چهارده', 'پانزده', 'شانزده', 'هفده', 'هجده', 'نوزده']
DIGITS_10_2 = ['',  '', 'بیست', 'سی', 'چهل', 'پنجاه', 'شصت', 'هفتاد', 'هشتاد', 'نود']
_DIGITS_100 = ['', ['صد', 'یکصد'], 'دویست', 'سیصد', 'چهارصد', 'پانصد', 'ششصد', 'هفتصد', 'هشتصد', 'نهصد']
DIGITS_100 = [x if isinstance(x, list) else [x] for x in _DIGITS_100]



from os import path

from setuptools import setup, find_packages

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), 'r+', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='fanumbers',
    version='1.0.1',
    description='Library to work with Farsi (Persian) numbers',
    author='AmirMohammad Dehghan',
    author_email='amirmd76@gmail.com',
    url='https://github.com/amirmd76/fanumbers',
    license='',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages()
)
